#!/usr/bin/perl
=head1 NAME

processSeq.pl - process sequences

=head1 SYNOPSIS

  processSeq.pl --pipeline 'Processor1i(-option,value)|Processor2' -isformat genbank  sequence_file

=head1 DESCRIPTION

This script is a general sequence (re)formatter and pipeline 'engine'. The modules passed as pipeline parameter
should implement Bio::Factory::SequenceProcessorI.


=head1 OPTIONS

  --help        this help page
  --pipeline    Pipe (|) separated list of processing modules.
                Example: 'Bio::Seq::SeqProcessor::RemoveSource|My::Other::Processor(-argument1,value1)'
  --verbose     Verbosity level, integer between -1 and 2. The verbosity level is passed to the pipeline modules as well.
                -1 = no warning
                0 = standard, small warning
                1 = warning with stack trace
                2 = warning becomes throw.
                Default: 0
  --isformat    Input sequence format supported by bioperl (fasta, genbank, ...).
                Default: fasta.
  --osformat    Output sequence format supported by bioperl.
                Default: genbank.

=head1 AUTHOR

Marc Logghe  E<lt>marc.logghe@ablynx.comE<gt>

=cut

use strict;
use Getopt::Long;
use Pod::Usage;

use Bio::SeqIO;

my $format = 'fasta';
my $outformat = 'genbank';
my $help;
my $verbose = 0;
my $pipeline;

my $ok = GetOptions(
  'help'       => \$help,
  'pipeline:s' => \$pipeline,
  'verbose:i'  => \$verbose,
  'isformat:s'   => \$format,
  'osformat:s'   => \$outformat
);
pod2usage(-verbose => 99, -sections => 'SYNOPSIS|OPTIONS') if ($help);
pod2usage(-verbose => 2) unless ( @ARGV );

  my $in = Bio::SeqIO->new(
    -format  => $format,
    -fh      => \*ARGV,
    -verbose => $verbose
  );


my @pipeline = split /\|/, $pipeline;
my @pipemods = setup_pipeline( \@pipeline, $verbose );

if (@pipemods)
{
  $pipemods[0]->source_stream($in);
  $in = $pipemods[-1];
}

my $out = Bio::SeqIO->new( -fh => \*STDOUT, -format => $outformat );

while (my $seq = $in->next_seq)
{
  $out->write_seq($seq);
}

sub setup_pipeline
{
  my ( $pipeline, $verbose ) = @_;
  my @pipemods = ();
  my @mods     = @$pipeline;

  # instantiate a module 'loader'
  my $loader = Bio::Root::Root->new();

  # load and instantiate each one, then concatenate
  foreach my $mod (@mods)
  {

    # separate module name from potential arguments
    my $modname = $mod;
    my @modargs = ();
    if ( $modname =~ /^(.+)[\(<](.*)[>\)]$/ )
    {
      $modname = $1;
      @modargs = split( /,/, $2 );
    }
    $loader->_load_module($modname);
    my $proc = $modname->new( -verbose => $verbose, @modargs );
    $loader->debug("Called '$modname->new(@modargs)'");
    if ( !$proc->isa("Bio::Factory::SequenceProcessorI") )
    {
      $loader->throw( "Pipeline processing module $modname does not implement "
          . 'Bio::Factory::SequenceProcessorI. Bummer.' );
    }
    $proc->source_stream( $pipemods[$#pipemods] ) if @pipemods;
    push( @pipemods, $proc );
  }
  return @pipemods;
}
